import express, { Request, Response } from 'express';
import { Account, Category, Transaction } from "../models";
import moment from "moment";
import { convertDateArray } from '../helper/dateConversion';

export class TransactionRouter {
    static routes() {
        let router = express.Router();
        router.use((req, res, next) => {
            console.log("TEST")
            next()
        })


        router.post('/addTransaction', async (req, res, next) => {
            console.log(req.body)

            try {
                /*
                if(
                    req.body.date == undefined
                    || req.body.Type == undefined
                    || req.body.CategoryID == undefined
                    || req.body.AccountID == undefined
                ){throw "ERROR"}
                    */
                let date = moment(req.body.date).set({
                    hour: req.body.time.split(":")[0],
                    minute:req.body.time.split(":")[1]
                })
                console.log(date.toDate())
                await Transaction.create({
                    Amount: req.body.amount,
                    Date: date.toDate(),
                    Type: req.body.type,
                    //@ts-ignore
                    UserID: 1, // Associate the transaction with the user
                    //@ts-ignore
                    CategoryID: req.body.category,
                    AccountID: req.body.account
                })
            } catch (e) {
                console.log(e)
            }  
            res.redirect("/transactions");
            //next();
        })
        router.post('/addTransfer', async (req, res, next) => {
            console.log(req.body)

            try {
                /*
                if(
                    req.body.date == undefined
                    || req.body.Type == undefined
                    || req.body.CategoryID == undefined
                    || req.body.AccountID == undefined
                ){throw "ERROR"}
                    */
                let date = moment(req.body.date).set({
                    hour: req.body.time.split(":")[0],
                    minute:req.body.time.split(":")[1]
                })
                console.log(date.toDate())
                await Transaction.transferAmountFromTo(1,req.body.fromAccount,req.body.toAccount,req.body.amount,req.body.categoryID)
                await Transaction.create({
                    Amount: req.body.amount,
                    Date: date.toDate(),
                    Type: req.body.type,
                    //@ts-ignore
                    UserID: 1, // Associate the transaction with the user
                    //@ts-ignore
                    CategoryID: req.body.category,
                    AccountID: req.body.account
                })
            } catch (e) {
                console.log(e)
            }  
            res.redirect("/transactions");
            //next();
        })
        router.get('/addTransaction', async (req, res) => {
            try {
                const transactions = await Transaction.findAll({
                    include: Account
                });
                const categories = await Category.findAll({order:[["Name","ASC"]]});
                const accounts = await Account.findAll({order:[["Name","ASC"]]});
                //console.log(transactions)
                res.render('transactions/formTransaction', { title: 'Transactions', transactions, categories, accounts });
            } catch (err) {
                console.error(err);
                res.status(500).send('Internal Server Error');
            }
        })
        router.get('/addTransfer', async (req, res) => {
            try {
                const transactions = await Transaction.findAll({
                    include: Account
                });
                const categories = await Category.findAll({order:[["Name","ASC"]]});
                const accounts = await Account.findAll({order:[["Name","ASC"]]});
                //console.log(transactions)
                res.render('transactions/formTransfer', { title: 'Transactions', transactions, categories, accounts });
            } catch (err) {
                console.error(err);
                res.status(500).send('Internal Server Error');
            }
        })

        router.get('/acc/:acc', async (req: Request, res: Response) => {
            try {
                const transactions = await Transaction.findAll({
                    include: [Account,Category],
                    where: {
                        AccountID: req.params.acc
                    }
                });
                let arr = convertDateArray(transactions);
                res.render('transactions/listTransactions', { title: 'Transactions', transactions:arr, cssClass:"" });
            } catch (err) {
                console.error(err);
                res.status(500).send('Internal Server Error');
            }
        });
        router.get('/cat/:cat', async (req: Request, res: Response) => {
            try {
                const transactions = await Transaction.findAll({
                    include: [Account, Category],
                    where: {
                        //@ts-ignore
                        CategoryID: req.params.cat
                    }
                });
                let arr = convertDateArray(transactions);
                res.render('transactions/listTransactions', { title: 'Transactions', transactions:arr, cssClass:"" });
            } catch (err) {
                console.error(err);
                res.status(500).send('Internal Server Error');
            }
        });
        router.all('/', async (req, res) => {
            try {
                let transactions:any = await Transaction.findAll({
                    include: [Account,Category]
                });
                const categories = await Category.findAll({});
                const accounts = await Account.findAll({});
                let arr = convertDateArray(transactions);
                console.log(transactions)
                res.render('transactions/listTransactions', { title: 'Transactions', transactions: arr, categories, accounts, cssClass:"fullList" });
            } catch (err) {
                console.error(err);
                res.status(500).send('Internal Server Error');
            }
        })

        return router;
    }
}